{
 "cells": [
  {
   "cell_type": "code",
   "execution_count": 10,
   "id": "57eb3a07",
   "metadata": {},
   "outputs": [],
   "source": [
    "import pandas as pd\n",
    "from glob import glob\n",
    "pd.options.display.float_format = '{:,.6f}'.format\n",
    "import pandas as pd\n",
    "from critdd import Diagram"
   ]
  },
  {
   "cell_type": "code",
   "execution_count": 2,
   "id": "008b8436",
   "metadata": {},
   "outputs": [
    {
     "data": {
      "text/plain": [
       "<Figure size 1500x1000 with 0 Axes>"
      ]
     },
     "metadata": {},
     "output_type": "display_data"
    }
   ],
   "source": [
    "import matplotlib.pyplot as plt\n",
    "from matplotlib.ticker import MaxNLocator\n",
    "from matplotlib import ticker\n",
    "%matplotlib inline\n",
    "import matplotlib\n",
    "import matplotlib.pylab as plt\n",
    "plt.rcParams[\"figure.figsize\"] = (10,7)\n",
    "fig = matplotlib.pyplot.gcf()\n",
    "fig.set_size_inches(10, 10, forward=True)\n",
    "fig.set_dpi(100)\n",
    "params = {'legend.fontsize': 'x-large',\n",
    "          'figure.figsize': (10, 10),\n",
    "         'axes.labelsize': 'x-large',\n",
    "         'axes.titlesize':'x-large',\n",
    "         'xtick.labelsize':'large',\n",
    "         'ytick.labelsize':'large'}\n",
    "plt.rcParams[\"figure.figsize\"] = (20,10)\n",
    "fig = matplotlib.pyplot.gcf()\n",
    "fig.set_size_inches(15, 10, forward=True)\n",
    "plt.rcParams.update(params)"
   ]
  },
  {
   "cell_type": "code",
   "execution_count": 3,
   "id": "75d53758",
   "metadata": {},
   "outputs": [],
   "source": [
    "class Tex:\n",
    "    def __init__(self, path):\n",
    "        self.raw_df=pd.read_pickle(path)\n",
    "        \n",
    "        self.automls=['Baseline', \n",
    "                      'MLJAR',\n",
    "                      'FLAML',\n",
    "                      'AutoSKLearn', \n",
    "                      'H2O',  \n",
    "                       'TPOT',\n",
    "                      'AutoGluon',\n",
    "                      'RSS']\n",
    "        \n",
    "        self.time_budgets=[30,\n",
    "                           60,\n",
    "                           120,\n",
    "                           180,\n",
    "                           300]\n",
    "        \n",
    "        assert(self.check())\n",
    "        \n",
    "    def __str__(self):\n",
    "        out='\\n'\n",
    "        for automl in self.automls:\n",
    "            out+=f'{df[df.automl==automl].head(1).to_markdown()}\\n'\n",
    "        return out\n",
    "    \n",
    "    def check(self):\n",
    "        df=self.raw_df\n",
    "        for a in df.automl.unique():\n",
    "            assert(a in self.automls)\n",
    "        assert(len(df.automl.unique()==8))\n",
    "        assert(len(df.project.unique())==31)\n",
    "        assert(len(df.seed.unique())==10)\n",
    "        assert(len(df.time_budget.unique()==5))\n",
    "        return True\n",
    "    \n",
    "    def get_overall(self):\n",
    "        df=self.raw_df.copy()\n",
    "        temp_data={}\n",
    "        for automl in self.automls:\n",
    "            filter_df=df[df.automl==automl].groupby('time_budget').agg(['mean','std'])\n",
    "            row=[]\n",
    "            for i in filter_df.index:\n",
    "                score=filter_df.loc[i].score['mean']\n",
    "                std=filter_df.loc[i].score['std']\n",
    "                row.append(f' {score.round(4)} $\\pm$ {std.round(2)} ')\n",
    "            temp_data[f'{automl}']=row\n",
    "        out_df=pd.DataFrame.from_dict(temp_data)\n",
    "        out_df.index=self.time_budgets\n",
    "        tex= self._get_tex(out_df)\n",
    "        return tex, out_df\n",
    "    \n",
    "    def _get_tex(self,df):\n",
    "        return str(df.to_latex().replace('\\$',\n",
    "                                         '$').replace('\\\\textbackslash ',\n",
    "                                                      '\\\\').replace('\\{',\n",
    "                                                                    '{').replace('\\}',\n",
    "                                                                                 '}'))\n",
    "    \n",
    "    def save_time_budget_figure(self):\n",
    "        df=self.raw_df\n",
    "        ax=None\n",
    "        axs=[]\n",
    "        for time_budget in reversed(sorted(df.time_budget.unique())):\n",
    "            \n",
    "            df_draw=df[(df.time_budget==time_budget)][['automl',\n",
    "                                                       'project',\n",
    "                                                       'score']].groupby(['project',\n",
    "                                                                          'automl']).agg(['mean'])\n",
    "            \n",
    "            ax=df_draw.unstack().plot(ax=ax, rot=45, style=['r*-','b--','y^-', \n",
    "                                                            'g*', 'y*','r--',\n",
    "                                                            'bo-','g:'], linewidth=2.0)\n",
    "            ax.set_title(f'Time Budget:{time_budget}')\n",
    "            ax.set_xlabel('Dataset')\n",
    "            ax.set_ylabel('$R^2$')\n",
    "            legends=[_[1] for _ in df_draw.index ]\n",
    "            plt.legend(legends,fontsize=20)\n",
    "            plt.grid(visible=True, which='major', color='0.75', linestyle='-')\n",
    "            plt.savefig(f'fig_a_{time_budget}.jpg')\n",
    "            plt.show()\n",
    "            axs.append(ax)\n",
    "            ax=None\n",
    "        return axs  "
   ]
  },
  {
   "cell_type": "code",
   "execution_count": 7,
   "id": "61843bf0",
   "metadata": {},
   "outputs": [
    {
     "data": {
      "text/html": [
       "<div>\n",
       "<style scoped>\n",
       "    .dataframe tbody tr th:only-of-type {\n",
       "        vertical-align: middle;\n",
       "    }\n",
       "\n",
       "    .dataframe tbody tr th {\n",
       "        vertical-align: top;\n",
       "    }\n",
       "\n",
       "    .dataframe thead th {\n",
       "        text-align: right;\n",
       "    }\n",
       "</style>\n",
       "<table border=\"1\" class=\"dataframe\">\n",
       "  <thead>\n",
       "    <tr style=\"text-align: right;\">\n",
       "      <th></th>\n",
       "      <th>automl</th>\n",
       "      <th>project</th>\n",
       "      <th>seed</th>\n",
       "      <th>time_budget</th>\n",
       "      <th>score</th>\n",
       "    </tr>\n",
       "  </thead>\n",
       "  <tbody>\n",
       "    <tr>\n",
       "      <th>0</th>\n",
       "      <td>AutoGluon</td>\n",
       "      <td>361072</td>\n",
       "      <td>219</td>\n",
       "      <td>30</td>\n",
       "      <td>0.848995</td>\n",
       "    </tr>\n",
       "    <tr>\n",
       "      <th>1</th>\n",
       "      <td>AutoGluon</td>\n",
       "      <td>361072</td>\n",
       "      <td>219</td>\n",
       "      <td>60</td>\n",
       "      <td>0.986669</td>\n",
       "    </tr>\n",
       "    <tr>\n",
       "      <th>2</th>\n",
       "      <td>AutoGluon</td>\n",
       "      <td>361072</td>\n",
       "      <td>219</td>\n",
       "      <td>120</td>\n",
       "      <td>0.986655</td>\n",
       "    </tr>\n",
       "    <tr>\n",
       "      <th>3</th>\n",
       "      <td>AutoGluon</td>\n",
       "      <td>361072</td>\n",
       "      <td>219</td>\n",
       "      <td>180</td>\n",
       "      <td>0.987282</td>\n",
       "    </tr>\n",
       "    <tr>\n",
       "      <th>4</th>\n",
       "      <td>AutoGluon</td>\n",
       "      <td>361072</td>\n",
       "      <td>219</td>\n",
       "      <td>300</td>\n",
       "      <td>0.987274</td>\n",
       "    </tr>\n",
       "    <tr>\n",
       "      <th>...</th>\n",
       "      <td>...</td>\n",
       "      <td>...</td>\n",
       "      <td>...</td>\n",
       "      <td>...</td>\n",
       "      <td>...</td>\n",
       "    </tr>\n",
       "    <tr>\n",
       "      <th>12128</th>\n",
       "      <td>TPOT</td>\n",
       "      <td>361104</td>\n",
       "      <td>194</td>\n",
       "      <td>30</td>\n",
       "      <td>0.000000</td>\n",
       "    </tr>\n",
       "    <tr>\n",
       "      <th>12129</th>\n",
       "      <td>TPOT</td>\n",
       "      <td>361104</td>\n",
       "      <td>194</td>\n",
       "      <td>60</td>\n",
       "      <td>0.000000</td>\n",
       "    </tr>\n",
       "    <tr>\n",
       "      <th>12130</th>\n",
       "      <td>TPOT</td>\n",
       "      <td>361104</td>\n",
       "      <td>219</td>\n",
       "      <td>30</td>\n",
       "      <td>0.000000</td>\n",
       "    </tr>\n",
       "    <tr>\n",
       "      <th>12131</th>\n",
       "      <td>TPOT</td>\n",
       "      <td>361104</td>\n",
       "      <td>219</td>\n",
       "      <td>60</td>\n",
       "      <td>0.000000</td>\n",
       "    </tr>\n",
       "    <tr>\n",
       "      <th>12132</th>\n",
       "      <td>TPOT</td>\n",
       "      <td>361104</td>\n",
       "      <td>219</td>\n",
       "      <td>120</td>\n",
       "      <td>0.000000</td>\n",
       "    </tr>\n",
       "  </tbody>\n",
       "</table>\n",
       "<p>12133 rows × 5 columns</p>\n",
       "</div>"
      ],
      "text/plain": [
       "          automl  project  seed  time_budget    score\n",
       "0      AutoGluon   361072   219           30 0.848995\n",
       "1      AutoGluon   361072   219           60 0.986669\n",
       "2      AutoGluon   361072   219          120 0.986655\n",
       "3      AutoGluon   361072   219          180 0.987282\n",
       "4      AutoGluon   361072   219          300 0.987274\n",
       "...          ...      ...   ...          ...      ...\n",
       "12128       TPOT   361104   194           30 0.000000\n",
       "12129       TPOT   361104   194           60 0.000000\n",
       "12130       TPOT   361104   219           30 0.000000\n",
       "12131       TPOT   361104   219           60 0.000000\n",
       "12132       TPOT   361104   219          120 0.000000\n",
       "\n",
       "[12133 rows x 5 columns]"
      ]
     },
     "execution_count": 7,
     "metadata": {},
     "output_type": "execute_result"
    }
   ],
   "source": [
    "tex.raw_df"
   ]
  },
  {
   "cell_type": "code",
   "execution_count": 17,
   "id": "406b1d69",
   "metadata": {},
   "outputs": [
    {
     "data": {
      "text/plain": [
       "array([361072, 361073, 361074, 361075, 361076, 361077, 361078, 361079,\n",
       "       361080, 361081, 361082, 361083, 361084, 361085, 361086, 361087,\n",
       "       361088, 361089, 361090, 361092, 361093, 361094, 361095, 361096,\n",
       "       361097, 361098, 361099, 361101, 361102, 361103, 361104])"
      ]
     },
     "execution_count": 17,
     "metadata": {},
     "output_type": "execute_result"
    }
   ],
   "source": [
    "tex.raw_df.project.unique()"
   ]
  },
  {
   "cell_type": "code",
   "execution_count": 8,
   "id": "d16247b9",
   "metadata": {},
   "outputs": [
    {
     "data": {
      "text/plain": [
       "Index(['AutoGluon', 'AutoSKLearn', 'Baseline', 'FLAML', 'H2O', 'MLJAR', 'RSS',\n",
       "       'TPOT'],\n",
       "      dtype='object', name='automl')"
      ]
     },
     "execution_count": 8,
     "metadata": {},
     "output_type": "execute_result"
    }
   ],
   "source": [
    "df=tex.raw_df[['automl','project','score']]\n",
    "df_pivot = df.pivot_table(\n",
    "    index = \"project\",\n",
    "    columns = \"automl\",\n",
    "    values = 'score'\n",
    ")\n",
    "df_pivot = df_pivot.loc[:,~df_pivot.apply(lambda x: x.duplicated(),axis=1).all()].copy()\n",
    "df_pivot.columns"
   ]
  },
  {
   "cell_type": "code",
   "execution_count": 13,
   "id": "9174d36b",
   "metadata": {},
   "outputs": [],
   "source": [
    "# create a CD diagram from the Pandas DataFrame\n",
    "diagram = Diagram(\n",
    "    df_pivot.to_numpy(),\n",
    "    treatment_names = df_pivot.columns,\n",
    "    maximize_outcome = True\n",
    ")\n",
    "\n",
    "# inspect average ranks and groups of statistically indistinguishable treatments\n",
    "diagram.average_ranks # the average rank of each treatment\n",
    "diagram.get_groups(alpha=.05, adjustment=\"holm\")\n",
    "\n",
    "# export the diagram to a file\n",
    "diagram.to_file(\n",
    "    \"automl_cd.tex\",\n",
    "    alpha = .05,\n",
    "    adjustment = \"holm\",\n",
    "    reverse_x = True,\n",
    "    axis_options = {\"title\": \"AutoML Critical Difference (CD) Plot\"},\n",
    ")"
   ]
  },
  {
   "cell_type": "code",
   "execution_count": 14,
   "id": "23fec066",
   "metadata": {},
   "outputs": [
    {
     "name": "stdout",
     "output_type": "stream",
     "text": [
      "\\begin{tikzpicture}[\n",
      "  treatment line/.style={rounded corners=1.5pt, line cap=round, shorten >=1pt},\n",
      "  treatment label/.style={font=\\small},\n",
      "  group line/.style={ultra thick},\n",
      "]\n",
      "\n",
      "\\begin{axis}[\n",
      "  clip={false},\n",
      "  axis x line={center},\n",
      "  axis y line={none},\n",
      "  axis line style={-},\n",
      "  xmin={1},\n",
      "  ymax={0},\n",
      "  scale only axis={true},\n",
      "  width={\\axisdefaultwidth},\n",
      "  ticklabel style={anchor=south, yshift=1.3*\\pgfkeysvalueof{/pgfplots/major tick length}, font=\\small},\n",
      "  every tick/.style={draw=black},\n",
      "  major tick style={yshift=.5*\\pgfkeysvalueof{/pgfplots/major tick length}},\n",
      "  minor tick style={yshift=.5*\\pgfkeysvalueof{/pgfplots/minor tick length}},\n",
      "  title style={yshift=\\baselineskip},\n",
      "  xmax={8},\n",
      "  ymin={-5.5},\n",
      "  height={6\\baselineskip},\n",
      "  xtick={1,2,3,4,5,6,7,8},\n",
      "  minor x tick num={1},\n",
      "  x dir={reverse},\n",
      "  title={AutoML Critical Difference (CD) Plot},\n",
      "]\n",
      "\n",
      "\\draw[treatment line] ([yshift=-2pt] axis cs:1.3225806451612903, 0) |- (axis cs:0.6559139784946236, -2.0)\n",
      "  node[treatment label, anchor=west] {RSS};\n",
      "\\draw[treatment line] ([yshift=-2pt] axis cs:2.7096774193548385, 0) |- (axis cs:0.6559139784946236, -3.0)\n",
      "  node[treatment label, anchor=west] {FLAML};\n",
      "\\draw[treatment line] ([yshift=-2pt] axis cs:2.967741935483871, 0) |- (axis cs:0.6559139784946236, -4.0)\n",
      "  node[treatment label, anchor=west] {H2O};\n",
      "\\draw[treatment line] ([yshift=-2pt] axis cs:5.387096774193548, 0) |- (axis cs:0.6559139784946236, -5.0)\n",
      "  node[treatment label, anchor=west] {AutoSKLearn};\n",
      "\\draw[treatment line] ([yshift=-2pt] axis cs:5.451612903225806, 0) |- (axis cs:6.827956989247312, -5.0)\n",
      "  node[treatment label, anchor=east] {AutoGluon};\n",
      "\\draw[treatment line] ([yshift=-2pt] axis cs:6.0, 0) |- (axis cs:6.827956989247312, -4.0)\n",
      "  node[treatment label, anchor=east] {Baseline};\n",
      "\\draw[treatment line] ([yshift=-2pt] axis cs:6.0, 0) |- (axis cs:6.827956989247312, -3.0)\n",
      "  node[treatment label, anchor=east] {TPOT};\n",
      "\\draw[treatment line] ([yshift=-2pt] axis cs:6.161290322580645, 0) |- (axis cs:6.827956989247312, -2.0)\n",
      "  node[treatment label, anchor=east] {MLJAR};\n",
      "\\draw[group line] (axis cs:5.387096774193548, -1.3333333333333333) -- (axis cs:6.161290322580645, -1.3333333333333333);\n",
      "\\draw[group line] (axis cs:2.7096774193548385, -2.0) -- (axis cs:2.967741935483871, -2.0);\n",
      "\n",
      "\\end{axis}\n",
      "\\end{tikzpicture}\n"
     ]
    }
   ],
   "source": [
    "print(open('automl_cd.tex').read())"
   ]
  },
  {
   "cell_type": "code",
   "execution_count": null,
   "id": "42b75bd5",
   "metadata": {},
   "outputs": [],
   "source": []
  },
  {
   "cell_type": "code",
   "execution_count": null,
   "id": "b8221357",
   "metadata": {},
   "outputs": [],
   "source": []
  },
  {
   "cell_type": "code",
   "execution_count": null,
   "id": "9a1ae2e6",
   "metadata": {},
   "outputs": [],
   "source": []
  },
  {
   "cell_type": "code",
   "execution_count": null,
   "id": "75051e82",
   "metadata": {},
   "outputs": [],
   "source": []
  },
  {
   "cell_type": "code",
   "execution_count": null,
   "id": "093e44a7",
   "metadata": {},
   "outputs": [],
   "source": []
  },
  {
   "cell_type": "code",
   "execution_count": null,
   "id": "eb9587ea",
   "metadata": {},
   "outputs": [],
   "source": []
  },
  {
   "cell_type": "code",
   "execution_count": null,
   "id": "45903cce",
   "metadata": {},
   "outputs": [],
   "source": []
  },
  {
   "cell_type": "code",
   "execution_count": null,
   "id": "3b0bb7b4",
   "metadata": {},
   "outputs": [],
   "source": []
  }
 ],
 "metadata": {
  "kernelspec": {
   "display_name": "Python 3 (ipykernel)",
   "language": "python",
   "name": "python3"
  },
  "language_info": {
   "codemirror_mode": {
    "name": "ipython",
    "version": 3
   },
   "file_extension": ".py",
   "mimetype": "text/x-python",
   "name": "python",
   "nbconvert_exporter": "python",
   "pygments_lexer": "ipython3",
   "version": "3.10.9"
  }
 },
 "nbformat": 4,
 "nbformat_minor": 5
}
